import { Pab } from "jalhyd";

import { FormulaireBase } from "./form-base";
import { FormComputePab } from "../form-compute-pab";
import { FormResultPab } from "../form-result-pab";

/**
 * Formulaire pour les passes à bassins, inspiré du formulaire
 * pour les structures en parallèle
 */
export class FormulairePab extends FormulaireBase {

    /** id du select configurant le modèle de cloisons */
    // private _modeleCloisonsSelectId: string;

    /** id du select configurant le modèle de la cloison aval */
    // private _modeleCloisonAvalSelectId: string;

    constructor() {
        super();
        this._formResult = new FormResultPab(this);

        // remove obsolete observer set by super()
        this.removeObserver(this._formCompute);
        this._formCompute = new FormComputePab(this, (this._formResult as FormResultPab));
    }

    public get pabNub(): Pab {
        return this.currentNub as Pab;
    }

    public doCompute() {
        this.dumpPabStructure(this.pabNub);
        super.doCompute();
    }

    /* public get modeleCloisonsSelectId(): string {
        return this._modeleCloisonsSelectId;
    }

    public get modeleCloisonAvalSelectId(): string {
        return this._modeleCloisonAvalSelectId;
    } */

    /**
     * Creates a virgin PabCloisons when a new fieldset is added through the GUI,
     * to ensure consistency; this object is not related to any Cloisons
     */
    /* private createDummyPabCloisons(templ: FieldsetTemplate): Nub {
        const params = {};
        params["calcType"] = templ.calcTypeFromConfig;
        return this.createBassin(new Props(params));
    } */

    /**
     * ajoute un Nub PabCloisons
     * @param after position après laquelle insérer la structure, à la fin sinon
     */
    /* private addPabCloisons(st: PabCloisons, after?: number) {
        this.pabNub.addChild(st, after);
    } */

    /**
     * Asks JaLHyd to create a PabCloisons Nub as a child of the current Calculator Module
     * and return it; does not store it in the Session (for PabCloisons, not for Calculator Modules)
     * @param p properties for the new Nub
     */
    /* protected createBassin(p: Props): PabCloisons {
        return Session.getInstance().createNub(p, this.currentNub as Pab) as PabCloisons;
    } */

    /**
     * Replaces the current Nub in the current calculator module, with a new one built with properties "params"
     * @param params properties to build the new Nub (calcType, loiDebit...)
     */
    /* protected replaceNub(sn: PabCloisons, params: Props): Nub {
        const parent = (this.currentNub as Pab);
        const newBassin = this.createBassin(params);
        parent.replaceChildInplace(sn, newBassin);
        return newBassin;
    } */

    /**
     * Deleted the given child Nub in the current calculator module
     * @param params properties to build the new Nub (calcType, loiDebit...)
     */
    /* protected deleteNub(sn: PabCloisons) {
        const parent = (this.currentNub as PabCloisons);
        parent.deleteChild(parent.getIndexForChild(sn));
    } */
    /* public createFieldset(parent: FormulaireNode, json: {}, data?: {}, nub?: Nub): FieldSet {
        if (json["calcType"] === "PabCloisons") {
            // indice après lequel insérer le nouveau FieldSet
            const after = data["after"];

            const res: FieldSet = new FieldSet(parent);
            let n: Nub;
            if (nub) { // use existing Nub (build interface based on model)
                n = nub;
            } else {
                n = this.createDummyPabCloisons(data["template"]);
                this.addPabCloisons(n as PabCloisons, after);
            }
            res.setNub(n, false);

            if (after !== undefined) {
                parent.kids.splice(after + 1, 0, res);
            } else {
                parent.kids.push(res);
            }

            this.resetResults();

            return res;
        } else {
            return super.createFieldset(parent, json, data);
        }
    } */

    /* protected parseOptions(json: {}) {
        super.parseOptions(json);
        // id du select configurant les modèles de cloisons
        this._modeleCloisonsSelectId = this.getOption(json, "modeleCloisonsSelectId");
        // id du select configurant le modèle de cloison aval
        this._modeleCloisonAvalSelectId = this.getOption(json, "modeleCloisonAvalSelectId");
    } */

    /* public afterParseFieldset(fs: FieldSet) {
        // si le FieldSet contient le select de modèles de cloisons
        if (this._modeleCloisonsSelectId) {
            const node = fs.getFormulaireNodeById(this._modeleCloisonsSelectId);
            const sel = (node as SelectField);
            if (sel) {
                // on abonne le formulaire aux propriétés du FieldSet
                fs.properties.addObserver(this);
            }
        }
        // si le FieldSet contient le select du modèle de la cloison aval
        if (this._modeleCloisonAvalSelectId) {
            const node = fs.getFormulaireNodeById(this._modeleCloisonAvalSelectId);
            const sel = (node as SelectField);
            if (sel) {
                // on abonne le formulaire aux propriétés du FieldSet
                fs.properties.addObserver(this);
            }
        }
    } */

    /* protected completeParse(json: {}) {
        this.subscribeFieldsetContainer();
    } */

    /* private get fieldsetContainer(): FieldsetContainer {
        const n = this.getFormulaireNodeById("bassin_container");
        if (n === undefined || !(n instanceof FieldsetContainer)) {
            throw new Error("l'élément 'bassin_container' n'est pas du type FieldsetContainer");
        }
        return n as FieldsetContainer;
    } */

    /**
     * abonnement en tant qu'observateur du FieldsetContainer
     */
    /* private subscribeFieldsetContainer() {
        this.fieldsetContainer.addObserver(this);
    } */

    /**
     * abonnement en tant qu'observateur des NgParameter des FieldSet contenus dans le FieldsetContainer
     */
    /* private subscribeBasinFields(fs: FieldSet) {
        for (const n of fs.allFormElements) {
            if (n instanceof NgParameter || n instanceof SelectField) {
                n.addObserver(this);
            }
        }
    } */

    // interface Observer

    /* public update(sender: any, data: any) {

        super.update(sender, data);

        if (sender instanceof FieldsetContainer) {
            switch (data.action) {
                case "newFieldset":
                    this.reset();
                    this.subscribeBasinFields(data["fieldset"]);
            }
        } else if (sender instanceof FieldSet && data.action === "propertyChange") {
            switch (sender.id) {
                case "fs_bassin":
                    switch (data.name) {
                        case "modeleCloisons":
                            // change PabCloisons property "modeleCloisons" and reinit it
                            // with new Cloisons values
                            const newModeleUID = sender.properties.getPropValue("modeleCloisons");
                            (sender.nub as PabCloisons).setModel(newModeleUID);
                            // treat the fieldset as new to re-subscribe to Nub properties change events
                            this.afterParseFieldset(sender);
                            this.reset();
                            break;
                    }
                    break;

                case "fs_cloison_aval":
                    switch (data.name) {
                        case "modeleCloisonAval":
                            // change Pab property "modeleCloisonAval" and reinit it
                            // with new ParallelStructure values
                            const newModeleAvalUID = sender.properties.getPropValue("modeleCloisonAval");
                            this.pabNub.setDownWall(newModeleAvalUID);
                            // treat the fieldset as new to re-subscribe to Nub properties change events
                            this.afterParseFieldset(sender);
                            this.reset();
                            break;
                    }
                    break;
            }
        }
    } */

    // debug method
    private dumpPabStructure(pab: Pab) {
        console.log(`PAB: ${pab.uid}, ${pab.children.length} children`);
        for (const c of pab.children) {
            console.log(
                ` * child: ${c.uid}, based on ${c.properties.getPropValue("modeleCloisons")}`
                + ` (cote amont ${c.prms.Z1.singleValue}, longueur ${c.prms.LB.singleValue})`
            );
        }
        if (pab.downWall) {
            console.log(`+ downstream wall: ${pab.downWall.uid}`);
        }
    }
}
