import { Result, Pab } from "jalhyd";

import { FormulaireDefinition } from "./form-definition";
import { FormResultPab } from "./form-result-pab";
import { FormCompute } from "./form-compute";
import { NgParameter, ParamRadioConfig } from "../ngparam";

export class FormComputePab extends FormCompute {

    constructor(formBase: FormulaireDefinition, formResult: FormResultPab) {
        super(formBase, formResult);
    }

    protected get formResult(): FormResultPab {
        return this._formResult as FormResultPab;
    }

    private getVariatedParameter(): NgParameter {
        const res = this._formBase.getDisplayedParamFromState(ParamRadioConfig.VAR);
        if (res !== undefined) {
            return res;
        }

        const pms = this._formBase.getDisplayedParamListFromState(ParamRadioConfig.LINK);
        for (const p of pms) {
            if (p.paramDefinition.hasMultipleValues) {
                return p;
            }
        }
    }

    protected compute() {
        this.runNubCalc(this._formBase.currentNub);
        this.reaffectResultComponents();
    }

    protected reaffectResultComponents() {
        const pab: Pab = (this._formBase.currentNub as Pab);
        const computedParam: NgParameter = this.getComputedParameter();
        const varParam: NgParameter = this.getVariatedParameter();

        // résultat de calcul de la passe à bassins
        const pabr = this.formResult.pabResults;
        pabr.calculatedParameter = computedParam;
        pabr.result = pab.result;

        // résultat de chaque cloison
        const cr: Result[] = [];
        for (const c of pab.children) {
            cr.push(c.result);
        }
        pabr.cloisonsResults = cr,
        // résultat de la cloison aval
        pabr.cloisonAvalResults = pab.downWall.result;
        // cote aval de la passe
        pabr.Z2 = pab.prms.Z2.v;

        if (varParam) {
            pabr.variatedParameter = varParam;
            // pabr.update(false);
        }
    }

    private getComputedParameter(): NgParameter {
        return this._formBase.getDisplayedParamFromState(ParamRadioConfig.CAL);
    }
}
